<?php if (!defined('PmWiki')) exit();
/*  Copyright 2010 Hans Bracker.
    This file is abctunebook.php; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published
    by the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    
    This script is part of the Cookbook/AbcTuneBook recipe for creating 
    and editing tunes in abc notation, organised in groups as tune books.
    It uses fox.php and foxedit.php for posting and editing abc tunes,
    and the javascript abc editor  abcjs from Gregory Dyke and Paul Rosen
    (http://drawthedots.com/abcjs) for rendering the abc notation as a 
    musical score and midi track for instant preview.
    
*/

$RecipeInfo['AbcTunebook']['Version'] = '2016-01-08';

SDV($AbcTunebooks['default'], 'Tunebook');
SDV($AddTunePage, 'NewTune');

SDVA($Abcjs, array(
	'dir' => $PubDirUrl.'/abcjs',
	'base' => 'abcjs_base-min.js',
	'editor' => 'abc_editor.js',
	'plugin' => 'abc_plugin.js',
	'midi_program' => '1',
	'midi_qpm' => '160',
));

# (:abcjs:) directive will load abcjs plugin scripts even if page is not in a tunebook group.
Markup_e('abcjs', 'directives', '/\\(:abcjs:\\)/is', "AbcjsLoadPlugin(\$pagename)");

# otherwise abcjs will not be loaded and markup rules not be set
$name = PageVar($pagename,'$Name');
$group = PageVar($pagename,'$Group');
if (!in_array($group, $AbcTunebooks)) return;

# load abcjs plugin
if($action=='browse') {
	AbcjsLoadPlugin($pagename);
	Markup_e('X:abc', '>[=','/(?<!\\]\\]\n)(X:\\s*\\d+\\s*\n.*?)\n<:vspace>/s',"AbcTextMarkup(\$m[1])");
	Markup_e('#abc', '<X:abc',
  '/\\[\\[#(abc\\d*)\\]\\](.*?)\\[\\[#\\1end\\]\\]/is', "AbcTextMarkup(\$m[2],\$m[1])");
}

$FmtPV['$MidiTempo'] = '$GLOBALS[Abcjs]["midi_qpm"]';

function AbcjsLoadPlugin($pagename) {
	global $Abcjs, $AbcTunebooks, $HTMLFooterFmt, $HTMLHeaderFmt, $HTMLStylesFmt;
	$speed = PageTextVar($pagename, 'speed');
	if (!$speed) $speed = 1;
	$rhythm = PageTextVar($pagename, 'R');
	if(!$rythm) $rythm = 'default';
	switch($rhythm) {
		case 'hornpipe': $qpm=160; break;
		case 'polka': $qpm=180; break;
		case 'jig': $qpm=200; break;
		case 'reel': $qpm=220; break;
		case 'slide': $qpm=240; break;
		case 'strathspey': $qpm=180; break;
		case 'slip jig': $qpm=220; break;
		default: $qpm=180;
	} 
	$Abcjs['midi_qpm'] = $qpm*$speed;
	$HTMLStylesFmt['abctext'] = " @media print {.abctext, #titlebar {display:none;} }
		.abctext {font-size:smaller; background:#efefef; margin:0.5em 0; padding:5px; border:1px solid #ccc;}
		.toggleLink {float:right; margin-right:1em;}";
	$HTMLHeaderFmt['abcjsplugin'] = "
	<script src=\"{$Abcjs['dir']}/{$Abcjs['base']}\" type=\"text/javascript\"></script>
	<script src=\"{$Abcjs['dir']}/{$Abcjs['plugin']}\" type=\"text/javascript\"></script>
	<script type='text/javascript'>abc_plugin['render_before']=false; abc_plugin['midi_options']['qpm'] = \"{$Abcjs['midi_qpm']}\"; abc_plugin['midi_options']['program'] = '{$Abcjs['midi_program']}';</script>
	";
}

function AbcTextMarkup($text, $section=false) {
	static $cnt=0; $cnt++;
	$text = trim($text);
	$out = '';
	if ($section) {
		$editlnk = "{[foxedit $section 'edit' form=Site.FoxAbcTemplates#editabc]}";
		$cnt = substr($section,3);
	}
	else $editlink = '';
	$out = "<div id='abc$cnt' class='abc'><div id='abctop{$cnt}' class='rfloat noprint'>$editlnk</div>".Keep($text)."</div>\n\n";
	return $out;	
}

# load javascript abc editor
if (($action =='foxedit' && substr($_GET['section'],0,3)=='abc') || $name==$AddTunePage) {
	$HTMLHeaderFmt['abceditor'] = "
	<script src=\"{$Abcjs['dir']}/{$Abcjs['base']}\" type=\"text/javascript\"></script>
	<script src='{$Abcjs['dir']}/{$Abcjs['editor']}' type='text/javascript'></script>
	";
	$HTMLFooterFmt['abceditor'] = "
	<script type='text/javascript'>
		window.onload = function() {
			abc_editor = new ABCEditor('abc',{canvas_id:'canvas', midi_id:'midi', warnings_id:'warnings', midi_options:{program:{$Abcjs['midi_program']}, qpm:{$Abcjs['midi_qpm']} }, });
		}
	</script>
	";
}

# make pagename and set title from first T: in abc 
# when adding new tune page with Fox form
$FoxFilterFunctions['AbcTitle'] = 'FoxAbcFilter';
function FoxAbcFilter($pagename, $fx) {
	if (!preg_match('/^X:\\s*(\\d+)\\s?\n/s', $fx['abc'], $match)) 
		FoxAbort($pagename, "$[Error: please add an initial X: field with an index number]");	
	if (!preg_match('/^T:\\s*(.*?)$/m', $fx['abc'], $match))
		FoxAbort($pagename, "$[Error: please add a T: field with the tune's title]");
	$fx['title'] = trim($match[1]);
	$fx['target']	= MakePageName($pagename, $fx['title']);
	return $fx;
}

# for Fox abcimport form. Matches abc tunes in text, tunes get re-indexed, 
# each tune gets enclosed with anchors [[#abc<idx>]]..[[#abc<idx>end]]
$FoxFilterFunctions['AbcImport'] = 'FoxAbcImportFilter';
function FoxAbcImportFilter($pagename, $fx) {
	$fx['title'] = $fx['target'];
	$text = $fx['text']; 
	$abcpat = "/(X:\\s*(\\d+)\\s*\n.*?)\n\\s*(?=(\n|X:\\s*\\d+))/s";
	if(!preg_match_all($abcpat, $text, $matches)) return $fx;
	$m = $matches[1];
	foreach($m as $i => $t) {
		$j = $i+1;
		$new = preg_replace('/X:\\s*(\\d+)/s','X: '.$j ,$t);
		$new = '[[#abc'.$j.']]'."\n".$new."\n\n".'[[#abc'.$j.'end]]';
		$text = str_replace($t, $new, $text);
	}
	$fx['text'] = $text;
	return $fx;
}

# {$AbcTitle} for setting title with first T: in abc
$FmtPV['$AbcTitle'] = 'FirstAbcTitle($pn)';
function FirstAbcTitle($pagename) {
	$page = RetrieveAuthPage($pagename, 'read', false, READPAGE_CURRENT);
	if (!$page) return;
	return AbcTextField($page['text'], 'T');
}

$MarkupExpr['abcfield'] = 'AbcGetField($pagename, $args[0], $args[1])';

function AbcGetField($pagename, $section, $key) {
	$text = RetrieveAuthSection($pagename, $section);
	return AbcTextField($text, $key);
}

function AbcTextField($text, $k) {
	if (!preg_match('/^'.$k.':(.*?)$/im', $text, $match)) return;
	return trim($match[1]);	
} 

//EOF